package ch.sahits.game.openpatrician.model.city;

import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;
import java.util.Random;
import java.util.ResourceBundle;

import ch.sahits.game.openpatrician.model.product.AmountablePrice;
import ch.sahits.game.openpatrician.model.product.EWare;
import ch.sahits.game.openpatrician.util.PropertyLoader;
import ch.sahits.game.openpatrician.util.l10n.Locale;
/**
 * Implementation of the city model. The model of the city should only be instanciated once.
 * A city is unique. therefore equality can be tested by identity.
 * @author Andi Hotz, (c) Sahits GmbH, 2011
 * Created on Jan 18, 2011
 *
 */
abstract class City implements ICity {
	// TODO set the polulation structure based on the settings from the properties and the start year
	// TODO initialize the city with wares
	private final EWare[] effectiveProduction;
	private final EWare[] ineffectiveProduction;
	private final String name;
	private final EKontorType kontorType;
	private final Locale locale = Locale.getInstance();
	/** Store the amount of wares in the city in the ware specific sizes */
	private final HashMap<EWare, AmountablePrice<EWare>> wares = new HashMap<EWare, AmountablePrice<EWare>>();

	public City(String configFileName) throws IOException {
		Properties props = PropertyLoader.loadProperties(configFileName);
		if (props.getProperty("effectiveProduction")==null){
			throw new IOException("The property file "+configFileName+" does not contain the effectiveProduction property");
		}
		if (props.getProperty("ineffectiveProduction")==null){
			throw new IOException("The property file "+configFileName+" does not contain the ineffectiveProduction property");
		}
		if (props.getProperty("name")==null){
			throw new IOException("The property file "+configFileName+" does not contain the name property");
		}
		if (props.getProperty("kontorType")==null){
			throw new IOException("The property file "+configFileName+" does not contain the kontorType property");
		}
		if (props.getProperty("effectiveProduction").trim().length()>0){
			String[] wareNames = props.getProperty("effectiveProduction").split(",");
			this.effectiveProduction = new EWare[wareNames.length];
			for (int i = 0; i < wareNames.length; i++) {
				this.effectiveProduction[i]=EWare.valueOf(wareNames[i]);
			}
		} else {
			this.effectiveProduction=null;
		}
		if (props.getProperty("ineffectiveProduction").trim().length()>0){
			String[] wareNames = props.getProperty("ineffectiveProduction").split(",");
			this.ineffectiveProduction = new EWare[wareNames.length];
			for (int i = 0; i < wareNames.length; i++) {
				this.ineffectiveProduction[i]=EWare.valueOf(wareNames[i]);
			}
		} else {
			this.ineffectiveProduction=null;
		}
		String n = props.getProperty("name");
		ResourceBundle messages = ResourceBundle.getBundle("ModelMessages", locale.getCurrentLocal());
		this.name=messages.getString(n);
		this.kontorType = EKontorType.valueOf(props.getProperty("kontorType"));
		initPopulation(props);
		initWares();
	}
	/**
	 * Init the amount of wares available in the city
	 * This method is protected so it can be overriden by subclasses for testing
	 */
	protected void initWares() {
		// TODO initialize the wares in a more controlled way
		Random rnd = new Random(System.nanoTime());
		for (EWare ware : EWare.values()) {
			boolean hasWare = rnd.nextInt(7)%7!=0;
			if (hasWare){
				int amount = rnd.nextInt(159)+1; // 1..150
				addNewWare(ware, amount);
			}
		}
		
	}
	/**
	 * Initialize the population of the different classes based on the properties
	 * @param props
	 */
	private void initPopulation(Properties props) {
		// TODO this must be done based on properties and starting year
		Random rnd = new Random(System.nanoTime());
		int pop = 1000+(int)Math.abs(rnd.nextGaussian()*4000); // value between 1000 and 5000
		int diffPoor = (int) ((rnd.nextGaussian()-0.5)*7);
		int poor = (int)((60.0+diffPoor)/100*pop); // about 60% poor
		setPopulation(poor, EPopulationClass.POOR);
		int medium = (pop-poor)*2/3;
		int rich = pop-poor-medium;
		setPopulation(medium, EPopulationClass.MEDIUM);
		setPopulation(rich, EPopulationClass.RICH);
		
	}
	/** Holding the population split by population classes */
	private final HashMap<EPopulationClass, Integer> population = new HashMap<EPopulationClass, Integer>();
	/**
	 * Retrieve the total population
	 * @return
	 */
	@Override
	public int getPopulation() {
		int count = 0;
		for (Integer i : population.values()) {
			count += i;
		}
		return count;
	}
	/**
	 * Set the population count for a apopulation class
	 * @param population count
	 * @param popClass population class
	 */
	@Override
	public void setPopulation(int population, EPopulationClass popClass) {
		this.population.put(popClass,population);
	}
	/**
	 * Retrieve the population count for a class
	 * @param popclass population class
	 * @return
	 */
	@Override
	public int getPopulation(EPopulationClass popclass){
		return population.get(popclass);
	}
	@Override
	public String getName() {
		return name;
	}
	@Override
	public EKontorType getKontorType() {
		return kontorType;
	}
	
	
	
	@Override
	public AmountablePrice<EWare> getWare(EWare ware) {
		if (!wares.containsKey(ware)){
			addNewWare(ware,0);
		}
		AmountablePrice<EWare> ret = wares.get(ware);
		return ret;
	}
	/**
	 * Add a new ware to the wares list
	 * @param ware
	 */
	private void addNewWare(EWare ware, int amount){
		if (wares.containsKey(ware)){
			throw new IllegalStateException("Ware ("+ware.name()+") does already exist");
		}
		AmountablePrice<EWare> amountable = new AmountablePrice<EWare>();
		amountable.add(amount, 0);
		wares.put(ware, amountable);
	}
	/**
	 * {@inheritDoc}
	 * This method is thread save.
	 */
	@Override
	public int move(EWare ware, int amount) {
		synchronized (ware) { // There is still a problem with networked solution where the model of the city is replicated.
			AmountablePrice<EWare> amounted = getWare(ware);
			if (amount<0 && wares.containsKey(ware) && amounted!=null && -amount>amounted.getAmount()){ // remove more than is available
				amount = -amounted.getAmount();
			}
			if (amount<0 && !wares.containsKey(ware)){
				amount = 0;
			}
			if (!wares.containsKey(ware)){ // If the ware does not exist add it with amount 0
				throw new IllegalStateException("All wares should be initialized");
			}
			int available = getWare(ware).getAmount();
			if (amount>0){ // amount was bought
				amounted.add(amount, ware.computeSellPrice(available, amount));
			} else { // amount sold
				amounted.remove(amount);
			}
			return amount;
		}
	}
	
	/**
	 * Retrieve the wares that are produced efficiently
	 * @return
	 */
	@Override
	public EWare[] getEffectiveProduction() {
		return effectiveProduction;
	}
	/**
	 * Retrieve the wares that are produced inefficiently
	 * @return
	 */
	@Override
	public EWare[] getIneffectiveProduction() {
		return ineffectiveProduction;
	}
	
}
