package ch.sahits.game.openpatrician.model.ship;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import ch.sahits.game.openpatrician.model.product.AmountablePrice;
import ch.sahits.game.openpatrician.model.product.EWare;
import ch.sahits.game.openpatrician.model.product.IWare;
/**
 * This model represents a crayer.
 * @author Andi Hotz, (c) Sahits GmbH, 2011
 * Created on Nov 19, 2011
 *
 */
class Crayer implements ICrayer {
	/** Size in barrels */
	private int size = 350; // this is not final since the size changes by year and shipyard
	/** State of the design standard level */
	private EShipUpgrade upgradeLevel;
	/** Name of the ship */
	private String name;
	/** Store the wares loaded on the ship together with their amount. The amount is ware specific*/
	private Map<IWare,AmountablePrice<EWare>> loadedWare = new HashMap<IWare, AmountablePrice<EWare>>();
	
	/**
	 * Constructor initializing the name and the upgrade level
	 * @param upgradeLevel
	 * @param name
	 */
	public Crayer(EShipUpgrade upgradeLevel, String name) {
		super();
		this.upgradeLevel = upgradeLevel;
		this.name = name;
	}
	/**
	 * Constructor for a Crayer without any upgrade
	 * @param name
	 */
	public Crayer(String name) {
		this(EShipUpgrade.NONE,name);
	}

	@Override
	public int getUpgradeSpaceReduction(EShipUpgrade level){
		switch (level) {
		case NONE:
			return 0;
		case LEVEL1:
			return 90;
		case LEVEL2:
			return 190;
		default:
			return 0;
		}
	}
	

	@Override
	public int getSize() {
		return size;
	}

	@Override
	public int getCapacity() {
		return size-getUpgradeSpaceReduction(upgradeLevel)-getLoad();
	}

	@Override
	public boolean isUpgradable() {
		return upgradeLevel!=EShipUpgrade.LEVEL2;
	}

	@Override
	public void upgrade() {
		EShipUpgrade[] levels = EShipUpgrade.values();
		for (int i = 0; i < levels.length; i++) {
			if (levels[i]==upgradeLevel){
				upgradeLevel = levels[i+1];
				break; // only one update allowed
			}
		}

	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Set<IWare> getLoadedWares() {
		return loadedWare.keySet();
	}
	/**
	 * {@inheritDoc}
	 * This method is not thread safe as it is only intended to be accessed by one thread at a time
	 */
	@Override
	public int load(IWare ware, int amount, int avgPrice) {
		// FIXME
		final short sizeInBarrels = ware.getSizeAsBarrels();
		amount = Math.abs(amount);
		int cap = ensureCapacity(amount*sizeInBarrels);
		// Make sure that for wares in other sizes than barrels we only load
		// complete loads.
		if (cap!=amount && sizeInBarrels!=1){
			cap = (cap/sizeInBarrels)*sizeInBarrels;
		}
//		if (!loadedWare.containsKey(ware)){
//			AmountablePrice<EWare> available = new AmountablePrice<EWare>();
//			available.add(amount, avgPrice);
//			loadedWare.put(ware, available);
//		} else {
//			int alreadyLoaded = loadedWare.get(ware).getAmount();
			AmountablePrice<EWare> available = getWare((EWare) ware);
			available.add(cap/ware.getSizeAsBarrels(), avgPrice);
//		}
		return cap/sizeInBarrels;
	}
	/**
	 * Check if the amount can be loaded
	 * @param amount
	 * @return
	 */
	private int ensureCapacity(int amount) {
		return Math.min(amount, getCapacity());
	}


	/**
	 * {@inheritDoc}
	 * This method is not thread safe as it is only intended to be accessed by one thread at a time
	 */
	@Override
	public int unload(IWare ware, int amount) {
		if (!loadedWare.containsKey(ware)){
			return 0; // nothing to unload
		}
		amount = Math.abs(amount);
		// convert to barrels
		amount = amount*ware.getSizeAsBarrels();
		final int loaded = loadedWare.get(ware).getAmount()*ware.getSizeAsBarrels();
		int unloaded = Math.min(loaded, amount);
		if (unloaded==loaded){ // unloaded completely
			loadedWare.remove(ware); 
		} else {
			AmountablePrice<EWare> available = loadedWare.get(ware);
			available.remove(unloaded/ware.getSizeAsBarrels());
//			loadedWare.put(ware,loaded-unloaded); // reduce the amount
		}
		return unloaded/ware.getSizeAsBarrels();
	}

	@Override
	public int getDamage() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void damage(Object damage) {
		// TODO Auto-generated method stub

	}

	@Override
	public void repair() {
		// TODO Auto-generated method stub

	}


	@Override
	public void setName(String name) {
		this.name=name;
	}


	@Override
	public int getLoad() {
		int sum=0;
		for (Entry<IWare,AmountablePrice<EWare>> entry : loadedWare.entrySet()) {
			int amount = entry.getValue().getAmount();
			int barrelSize = entry.getKey().getSizeAsBarrels();
			sum += amount*barrelSize;
		}
		return sum;
	}


	/**
	 * Clear all loaded wares. This method is only intended for testing
	 */
	void clearLoadedWares(){
		loadedWare.clear();
	}
	@Override
	public AmountablePrice<EWare> getWare(EWare ware) {
		if (!loadedWare.containsKey(ware)){
			loadedWare.put(ware, new AmountablePrice<EWare>());
		}
		return loadedWare.get(ware);
	}

}
